// lpa_speech.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include <strstream>
#include "sapi.h"
#include <string.h>
#include <stdio.h>
#include <atlbase.h>
#include <windows.h>
#include "sphelper.h"
#include <iostream>

extern "C" __declspec(dllexport) void lpa_speak_no_attr(char * mystring);
extern "C" __declspec(dllexport) char * lpa_listen();
extern "C" __declspec(dllexport) void lpa_speak(char * mystring, int rate, int volume, char * required, char * optional);

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

inline HRESULT BlockForResult(ISpRecoContext * pRecoCtxt, ISpRecoResult ** ppResult)
{
    HRESULT hr = S_OK;
	CSpEvent event;

    while (SUCCEEDED(hr) &&
           SUCCEEDED(hr = event.GetFrom(pRecoCtxt)) &&
           hr == S_FALSE)
    {
        hr = pRecoCtxt->WaitForNotifyEvent(INFINITE);
    }

    *ppResult = event.RecoResult();
    if (*ppResult)
    {
        (*ppResult)->AddRef();
    }

    return hr;
}

// lpa_speak_no_attr(+Text)
//  This function speaks a line of text using the default voice settings as set by the user.
void lpa_speak_no_attr(char * mystring)
{
    ISpVoice * pVoice = NULL;

    if (FAILED(::CoInitialize(NULL)))
        return;

    HRESULT hr = CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&pVoice);
    if( SUCCEEDED( hr ) )
    {
		hr = pVoice->Speak((CSpDynamicString)mystring, 0, NULL);
        pVoice->Release();
        pVoice = NULL;
    }

    ::CoUninitialize();
}

// lpa_listen(-Text)
//  This function uses SAPI's speech recognition to listen for and return a line of text.
char * lpa_listen()
{
	HRESULT hr;
    if (SUCCEEDED(hr = ::CoInitialize(NULL)))
    {
        {
            CComPtr<ISpRecoContext> cpRecoCtxt;
            CComPtr<ISpRecoGrammar> cpGrammar;
            CComPtr<ISpVoice> cpVoice;
            hr = cpRecoCtxt.CoCreateInstance(CLSID_SpSharedRecoContext);
            if (cpRecoCtxt &&
                SUCCEEDED(hr = cpRecoCtxt->SetNotifyWin32Event()) &&
                SUCCEEDED(hr = cpRecoCtxt->SetInterest(SPFEI(SPEI_RECOGNITION), SPFEI(SPEI_RECOGNITION))) &&
                SUCCEEDED(hr = cpRecoCtxt->SetAudioOptions(SPAO_RETAIN_AUDIO, NULL, NULL)) &&
                SUCCEEDED(hr = cpRecoCtxt->CreateGrammar(0, &cpGrammar)) &&
                SUCCEEDED(hr = cpGrammar->LoadDictation(NULL, SPLO_STATIC)) &&
                SUCCEEDED(hr = cpGrammar->SetDictationState(SPRS_ACTIVE)))
            {
                CComPtr<ISpRecoResult> cpResult;

                while (SUCCEEDED(hr = BlockForResult(cpRecoCtxt, &cpResult)))
                {
                    cpGrammar->SetDictationState( SPRS_INACTIVE );

                    CSpDynamicString dstrText;
					char * mystring;

                    if (SUCCEEDED(cpResult->GetText(SP_GETWHOLEPHRASE, SP_GETWHOLEPHRASE, 
                                                    TRUE, &dstrText, NULL)))
                    {
						mystring = dstrText.CopyToChar();
                       cpResult.Release();
					   return mystring;
                    }
					cpGrammar->SetDictationState( SPRS_ACTIVE );
                } 
            }
        }
        ::CoUninitialize();

	}
}

// lpa_speak(+Text, +Rate, +Volume, +ReqAttributes, +OptAttributes)
//  This function uses SpFindBestToken to search for a voice that best matches
//  the input attributes and uses that voice to speak the line of text.
void lpa_speak(char * mystring, int rate, int volume, char * required, char * optional)
{
    ISpVoice * pVoice = NULL;
	ISpObjectToken * pCurVoiceToken;
	WCHAR * wrequired, * woptional;
	HRESULT hr = TRUE;

    if (FAILED(::CoInitialize(NULL)))
        return;

	wrequired = ((CSpDynamicString)required).Copy();

	woptional = ((CSpDynamicString)optional).Copy();

	hr = SpFindBestToken(SPCAT_VOICES, wrequired, woptional, &pCurVoiceToken);

	hr = CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&pVoice);

    if( SUCCEEDED( hr ) )
    {
		hr = pVoice->SetVoice(pCurVoiceToken);
		if((rate < -10) || (rate > 10))
		{
			rate = 0;
		};
		if((volume < 0) || (volume > 100))
		{
			volume = 50;
		};
		hr = pVoice->SetRate(rate);
		hr = pVoice->SetVolume(volume);
        hr = pVoice->Speak((CSpDynamicString)mystring, 0, NULL);
        pVoice->Release();
        pVoice = NULL;
    }

    ::CoUninitialize();

}